'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (ComponentStyle) {
  var createStyledComponent = function createStyledComponent(target, rules, styleProps) {
    var componentStyle = new ComponentStyle(rules);

    return {
      functional: true,
      render: function render(createElement, context) {
        var _ref = context || {},
            children = _ref.children,
            data = _ref.data;

        var props = Object.assign(data && data.attrs ? data.attrs : {}, styleProps);

        Object.keys(props).forEach(function (index) {
          if (typeof props[index] === 'function') {
            props[index] = props[index](props);
          }
        });

        var className = componentStyle.generateAndInjectStyles(props);

        if (context) {
          return createElement(target, Object.assign(context, {
            class: className,
            domProps: { value: data.model ? data.model.value : undefined },
            on: {
              input: function input(event) {
                if (data.model) data.model.callback(event.target.value);
              }
            }
          }), children);
        }
      },
      extend: function extend() {
        for (var _len = arguments.length, extendedRules = Array(_len), _key = 0; _key < _len; _key++) {
          extendedRules[_key] = arguments[_key];
        }

        var extended = [];

        extendedRules[0].forEach(function (line, key) {
          extended.push(line);
          extended.push(extendedRules[key + 1]);
        });

        return createStyledComponent(target, rules.slice().concat(extended), styleProps);
      },
      withComponent: function withComponent(newTarget) {
        return createStyledComponent(newTarget, rules, styleProps);
      },

      target: target
    };
  };
  return createStyledComponent;
};

module.exports = exports['default'];